<?php
/**
 *
 * @package RelatedTopics
 * @license http://madjoki.com/smf-mods/license/ New-BSD
 *
 * Converted to ElkArte:
 * @convertor  	ahrasis ahrasis@gmail http://elkarte.sch.my (ElkArte Version)
 *
 */

if (!defined('ELK'))
	die('No access...');

/**
 * RelatedTopics_Main()
 *
 * - Display Related Topics
 *
**/
function RelatedTopics_Main()
{
	global $modSettings, $context, $txt;

	if (!empty($modSettings['relatedTopicsEnabled']))
	{
		// Define template layers
		$template_layers = Template_Layers::getInstance();
		// Load the template
		loadTemplate('addons/RelatedTopics');
		// Load Language
		loadLanguage('addons/RelatedTopics');
		// Add QRR under the last post
		$template_layers->addAfter('RelatedTopics', 'pages_and_buttons');
		// Fix QRR css
		$context['html_headers'] .= '';
	}
}

/**
 * RelatedTopics_Create()
 *
 * - Create Related Topics
 *
**/
function RelatedTopics_Create(&$topicOptions)
{
	global $modSettings;

	if (!empty($modSettings['relatedTopicsEnabled']))
	{
		require_once(SOURCEDIR . '/addons/RelatedTopics.Subs.php');
		relatedUpdateTopics($topicOptions['id']);
	}
}

/**
 * RelatedTopics_Modify()
 *
 * - Modify Related Topics
 *
**/
function RelatedTopics_Modify(&$msgOptions, &$topicOptions)
{
	global $modSettings;

	if (!empty($modSettings['relatedTopicsEnabled']) && isset($msgOptions['subject']))
	{
		// Only update the subject if this was the first message in the topic.
		$request = $db->query('', '
			SELECT id_topic
			FROM {db_prefix}topics
			WHERE id_first_msg = {int:id_first_msg}
			LIMIT 1',
			array(
				'id_first_msg' => $msgOptions['id'],
			)
		);
		if ($db->num_rows($request) == 1)
		{
			updateStats('subject', $topicOptions['id'], $msgOptions['subject']);

			// Added by Related Topics
			require_once(SOURCEDIR . '/addons/RelatedTopics.Subs.php');
			relatedUpdateTopics($topicOptions['id']);
			// Related Topics END
		}
		$db->free_result($request);
	}
}

/**
 * RelatedTopics_Remove()
 *
 * - Remove Related Topics
 *
**/
function RelatedTopics_Remove($topics)
{
	global $modSettings;

	if (!empty($modSettings['relatedTopicsEnabled']))
	{
		require_once(SOURCEDIR . '/addons/RelatedTopics.Subs.php');
		relatedRemoveTopics($topics);
	}
}

/**
 * RelatedTopics_Load()
 *
 * - Load Related Topics
 *
**/
function RelatedTopics_Load()
{
	global $modSettings, $topic;

	if (!empty($modSettings['relatedTopicsEnabled']))
	{
		require_once(SOURCEDIR . '/addons/RelatedTopics.Subs.php');
		loadRelated($topic);
	}
}

/**
 * RelatedTopicsAdmin()
 *
 * - Related Topics Admin controller?
 *
**/
function RelatedTopicsAdmin()
{
	global $context, $txt;

	require_once(SOURCEDIR . '/addons/RelatedTopics.Subs.php');
	require_once(ADMINDIR . '/ManageServer.controller.php');

	$related_version = '1.4';
	// Load Template
	loadTemplate('addons/RelatedTopicsAdmin');
	// Load Language
	loadLanguage('addons/RelatedTopics');

	$context[$context['admin_menu_name']]['tab_data']['title'] = $txt['related_topics_admin_title'];
	$context[$context['admin_menu_name']]['tab_data']['description'] = $txt['related_topics_admin_desc'];

	$context['page_title'] = $txt['related_topics_admin_title'];

	$subActions = array(
		'main' => array('RelatedTopics_AdminTemplate'),
		'settings' => array('RelatedTopics_Settings'),
		'methods' => array('RelatedTopics_Methods'),
		'buildIndex' => array('RelatedTopics_Rebuilds'),
	);

	$_REQUEST['sa'] = isset($_REQUEST['sa']) && isset($subActions[$_REQUEST['sa']]) ? $_REQUEST['sa'] : 'main';

	if (isset($subActions[$_REQUEST['sa']][1]))
		isAllowedTo($subActions[$_REQUEST['sa']][1]);

	$subActions[$_REQUEST['sa']][0]();
}

/**
 * RelatedTopics Admin Template
 *
**/
function RelatedTopics_AdminTemplate()
{
	global $context, $txt;

	// Load Language
	loadLanguage('addons/RelatedTopics');

	$context['sub_template'] = 'related_topics_admin_main';
}

/**
 * RelatedTopics_AdminMenu()
 *
 * - Admin Hook, integrate_admin_areas, called from Admin.controller.php
 * - Used to add/modify admin menu areas
 *
 * @param mixed[] $admin_areas
**/
function RelatedTopics_Menu(&$admin_areas)
{
	global $txt;

	// Load Language
	loadLanguage('addons/RelatedTopics');

	$admin_areas['config']['areas']['relatedtopics'] = array(
		'label' => 'Related Topics',
		'dir' => SOURCEDIR . '/addons',
		'file' => 'RelatedTopics.php',
		'function' => 'RelatedTopicsAdmin',
		'icon' => 'transparent.png',
		'permission' => 'admin_forum',
		'subsections' => array(
			'main' => 'Information',
			'settings' => 'Settings',
			'methods' => 'Methods',
		)
	);
}

/**
 * AlternativeMenu_Subactions()
 *
 * - Admin Hook, integrate_modify_modifications, called from AddonSettings.controller.php
 * - Used to add subactions to the modification area
 *
 * @param mixed[] $sub_actions
**/
function RelatedTopics_Subactions(&$subActions)
{
	loadLanguage('addons/RelatedTopics');

	$sub_actions['relatedtopics'] = array(
		'dir' => SOURCEDIR . '/addons',
		'file' => 'RelativeTopics.php',
		'function' => 'RelatedTopics_Settings',
		'permission' => 'admin_forum',
	);
}
/**
 * RelatedTopics_Settings()
 *
 * - Defines our settings array and uses our settings class to manage the data
 */
function RelatedTopics_Settings($return_config = false)
{
	global $txt, $context, $scripturl, $modSettings;

	// Lets build a settings form
	require_once(SUBSDIR . '/SettingsForm.class.php');
	
	// Load languages
	loadLanguage('addons/RelatedTopics');
	
	// Initialize it with our settings
	$config_vars = array(
		array('check', 'relatedTopicsEnabled'),
		array('int', 'relatedTopicsCount'),
	);

	// Saving?
	if (isset($_GET['save']))
	{
		checkSession();
		Settings_Form::save_db($config_vars);
		writeLog();
		redirectexit('action=admin;area=relatedtopics;sa=settings');
	}
	
	// Prepare page $context
	$context['settings_title'] = $txt['admin_related_topics_settings'];
	$context['page_title'] = $txt['related_topics_admin_title'] . ' - ' . $txt['admin_related_topics_settings'];
	$context['sub_template'] = 'show_settings';
	$context['post_url'] = $scripturl . '?action=admin;area=relatedtopics;save;sa=settings;';

	Settings_Form::prepare_db($config_vars);
}

/**
 * RelatedTopics_Methods()
 *
 * - Methods to determine Related Topics
 *
**/
function RelatedTopics_Methods()
{
	global $context, $modSettings, $scripturl, $user_info, $txt, $db_type;
	$db = database();
	
	initRelated();

	$relatedIndexes = !empty($modSettings['relatedIndex']) ? explode(',', $modSettings['relatedIndex']) : array();

	$context['related_methods'] = array(
		'fulltext' => array(
			'name' => $txt['relatedFulltext'],
			'selected' => false,
			'supported' => $db_type == 'mysql',
		),
	);

	foreach ($context['related_methods'] as $id => $dummy)
		$context['related_methods'][$id]['selected'] = in_array($id, $relatedIndexes);

	if (isset($_GET['save']))
	{
		checkSession('post');

		$methods = array();

		if (isset($_POST['related_methods']))
		{
			foreach ($_POST['related_methods'] as $method)
			{
				if (isset($context['related_methods'][$method]) && $context['related_methods'][$method]['supported'])
					$methods[] = $method;
			}
		}

		updateSettings(array(
			'relatedIndex' => implode(',', $methods),
			'relatedIgnoredboards' => !empty($_POST['ignored_boards']) ? implode(',', $_POST['ignored_boards']) : '',
		));
		
		redirectexit('action=admin;area=relatedtopics;sa=methods');
	}

	$request = $db->query('', '
		SELECT b.id_board, b.name, c.id_cat, c.name AS cat_name
		FROM {db_prefix}boards AS b
			LEFT JOIN {db_prefix}categories AS c ON (c.id_cat = b.id_cat)
		WHERE b.redirect = {string:blank_redirect}'. (!empty($modSettings['recycle_enable']) && !empty($modSettings['recycle_board']) ? '
			AND NOT b.id_board = {int:recyle_board}' : ''),
		array(
			'blank_redirect' => '',
			'recyle_board' => $modSettings['recycle_board'],
		)
	);
	
	$context['categories'] = array();
	
	while ($row = $db->fetch_assoc($request))
	{
		if (!isset($context['categories'][$row['id_cat']]))
			$context['categories'][$row['id_cat']] = array(
				'id' => $row['id_cat'],
				'name' => $row['cat_name'],
				'boards' => array(),
			);
		
		$context['categories'][$row['id_cat']]['boards'][$row['id_board']] = array(
			'id' => $row['id_board'],
			'name' => $row['name'],
			'selected' => in_array($row['id_board'], $context['rt_ignore']),
		);
	}
	$db->free_result($request);

	$context['sub_template'] = 'related_topics_admin_methods';
}

/**
 * RelatedTopics_Rebuilds()
 *
 * - (Re)build Related Topics Index
 *
**/
function RelatedTopics_Rebuilds()
{
	global $scripturl, $modSettings, $context, $txt;
	$db = database();

	loadTemplate('Admin');
	loadLanguage('Admin');

	if (!isset($context['relatedClass']) && !initRelated())
		fatal_lang_error('no_methods_selected');
		
	$context['step'] = empty($_REQUEST['step']) ? 0 : (int) $_REQUEST['step'];

	if ($context['step'] == 0)
	{
		// Clear caches
		foreach ($context['relatedClass'] as $class)
			$class->recreateIndexTables();

		$db->query('', '
			DELETE FROM {db_prefix}related_topics'
		);
	}

	$request = $db->query('', '
		SELECT MAX(id_topic)
		FROM {db_prefix}topics');
	list ($max_topics) = $db->fetch_row($request);
	$db->free_result($request);

	// How many topics to do per page load?
	$perStep = 150;
	$last = $context['step'] + $perStep;

	// Search for topic ids between first and last which are not in ignored boards
	$request = $db->query('', '
		SELECT t.id_topic
		FROM {db_prefix}topics AS t
		WHERE t.id_topic > {int:start}
			AND t.id_topic <= {int:last}' . (!empty($context['rt_ignore']) ? '
			AND t.id_board NOT IN({array_int:ignored})' : ''),
		array(
			'start' => $context['step'],
			'last' => $last,
			'ignored' => $context['rt_ignore'],
		)
	);

	$topics = array();

	while ($row =  $db->fetch_assoc($request))
		$topics[] = $row['id_topic'];
	$db->free_result($request);

	// Update topics
	relatedUpdateTopics($topics, true);

	if ($last >= $max_topics)
		redirectexit('action=admin;area=relatedtopics;sa=methods');

	$context['sub_template'] = 'not_done';
	$context['continue_get_data'] = '?action=admin;area=relatedtopics;sa=buildIndex;step=' . $last;

	$context['continue_percent'] = round(100 * ($last / $max_topics));
	$context['continue_post_data'] = '';
	$context['continue_countdown'] = '2';
	
	obExit();
}

/**
 * Action Array()
 *
 * - Create Related Topics
 *
**/
function RelatedTopics_Action(&$actionArray)
{
	global $modSettings;

	if (!empty($modSettings['relatedTopicsEnabled']))
	{
		$actionArray['related'] = array('RelatedTopics.Subs.php', 'Related');
	}
}

?>